const char httpc_lua[] =
"--\n"
"--  Copyright (C) 2016-2017 Tarantool AUTHORS: please see AUTHORS file.\n"
"--\n"
"--  Redistribution and use in source and binary forms, with or\n"
"--  without modification, are permitted provided that the following\n"
"--  conditions are met:\n"
"--\n"
"--  1. Redistributions of source code must retain the above\n"
"--   copyright notice, this list of conditions and the\n"
"--   following disclaimer.\n"
"--\n"
"--  2. Redistributions in binary form must reproduce the above\n"
"--   copyright notice, this list of conditions and the following\n"
"--   disclaimer in the documentation and/or other materials\n"
"--   provided with the distribution.\n"
"--\n"
"--  THIS SOFTWARE IS PROVIDED BY <COPYRIGHT HOLDER> ``AS IS'' AND\n"
"--  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED\n"
"--  TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR\n"
"--  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL\n"
"--  <COPYRIGHT HOLDER> OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,\n"
"--  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL\n"
"--  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF\n"
"--  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR\n"
"--  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF\n"
"--  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT\n"
"--  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF\n"
"--  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF\n"
"--  SUCH DAMAGE.\n"
"--\n"
"\n"
"local fiber = require('fiber')\n"
"local driver = require('http.client.driver')\n"
"\n"
"local curl_mt\n"
"\n"
"--\n"
"--  <http> - create a new curl instance.\n"
"--\n"
"--  Parameters:\n"
"--\n"
"--  max_connectionss -  Maximum number of entries in the connection cache */\n"
"--\n"
"--  Returns:\n"
"--  curl object or raise error()\n"
"--\n"
"\n"
"local http_new = function(opts)\n"
"\n"
"    opts = opts or {}\n"
"\n"
"    opts.max_connections = opts.max_connections or 5\n"
"\n"
"    local curl = driver.new(opts.max_connections)\n"
"    return setmetatable({ curl = curl, }, curl_mt )\n"
"end\n"
"\n"
"local check_args_fmt = 'Use client:%s(...) instead of client.%s(...):'\n"
"\n"
"local function check_args(self, method)\n"
"    if type(self) ~= 'table' then\n"
"        error(check_args_fmt:format(method, method), 2)\n"
"    end\n"
"end\n"
"\n"
"local function parse_list(list)\n"
"    local result = {}\n"
"    for _,str in pairs(list) do\n"
"        if str ~= '' and not string.match(str, \"HTTP/%d%.%d %d%d%d\") then\n"
"            local h = str:split(': ')\n"
"            local key = table.remove(h, 1)\n"
"            local val = table.concat(h)\n"
"            result[string.lower(key)] = val\n"
"        end\n"
"    end\n"
"    return result\n"
"end\n"
"\n"
"local function parse_headers(resp)\n"
"    local list = resp.headers:split('\\r\\n')\n"
"    local h1 = table.remove(list, 1):split(' ')\n"
"    local proto = h1[1]:split('/')[2]:split('.')\n"
"    resp.proto = { tonumber(proto[1]), tonumber(proto[2]) }\n"
"    resp.headers = parse_list(list)\n"
"    return resp\n"
"end\n"
"\n"
"--\n"
"--  <request> This function does HTTP request\n"
"--\n"
"--  Parameters:\n"
"--\n"
"--  method  - HTTP method, like GET, POST, PUT and so on\n"
"--  url     - HTTP url, like https://tarantool.org/doc\n"
"--  body    - this parameter is optional, you may use it for passing\n"
"--  options - this is a table of options.\n"
"--       data to a server. Like 'My text string!'\n"
"--\n"
"--      ca_path - a path to ssl certificate dir;\n"
"--\n"
"--      ca_file - a path to ssl certificate file;\n"
"--\n"
"--      verify_host - set on/off verification of the certificate's name (CN)\n"
"--          against host;\n"
"--\n"
"--      verify_peer - set on/off verification of the peer's SSL certificate;\n"
"--\n"
"--      ssl_key - set path to the file with private key for TLS and SSL client\n"
"--          certificate;\n"
"--\n"
"--      ssl_cert - set path to the file with SSL client certificate;\n"
"--\n"
"--      headers - a table of HTTP headers;\n"
"--\n"
"--      keepalive_idle & keepalive_interval -\n"
"--          non-universal keepalive knobs (Linux, AIX, HP-UX, more);\n"
"--\n"
"--      low_speed_time & low_speed_limit -\n"
"--          If the download receives less than\n"
"--          \"low speed limit\" bytes/second\n"
"--          during \"low speed time\" seconds,\n"
"--          the operations is aborted.\n"
"--          You could i.e if you have\n"
"--          a pretty high speed connection, abort if\n"
"--          it is less than 2000 bytes/sec\n"
"--          during 20 seconds;\n"
"--\n"
"--      timeout - Time-out the read operation and\n"
"--          waiting for the curl api request\n"
"--          after this amount of seconds;\n"
"--\n"
"--      verbose - set on/off verbose mode\n"
"--\n"
"--  Returns:\n"
"--      {\n"
"--          status=NUMBER,\n"
"--          reason=ERRMSG\n"
"--          body=STRING,\n"
"--          headers=STRING,\n"
"--          errmsg=STRING\n"
"--      }\n"
"--\n"
"--  Raises error() on invalid arguments and OOM\n"
"--\n"
"\n"
"curl_mt = {\n"
"    __index = {\n"
"        --\n"
"        --  <request> see above <request>\n"
"        --\n"
"        request = function(self, method, url, body, opts)\n"
"            if not method or not url then\n"
"                error('request(method, url [, options]])')\n"
"            end\n"
"            local resp = self.curl:request(method, url, body, opts or {})\n"
"            if resp and resp.headers then\n"
"                resp = parse_headers(resp)\n"
"            end\n"
"            return resp\n"
"        end,\n"
"\n"
"        --\n"
"        -- <get> - see <request>\n"
"        --\n"
"        get = function(self, url, options)\n"
"            check_args(self, 'get')\n"
"            return self:request('GET', url, nil, options)\n"
"        end,\n"
"\n"
"        --\n"
"        -- <post> - see <request>\n"
"        --\n"
"        post = function(self, url, body, options)\n"
"            check_args(self, 'post')\n"
"            return self:request('POST', url, body, options)\n"
"        end,\n"
"\n"
"        --\n"
"        -- <put> - see <request>\n"
"        --\n"
"        put = function(self, url, body, options)\n"
"            check_args(self, 'put')\n"
"            return self:request('PUT', url, body, options)\n"
"        end,\n"
"\n"
"        --\n"
"        -- <patch> - see <request>\n"
"        --\n"
"        patch = function(self, url, body, options)\n"
"            check_args(self, 'patch')\n"
"            return self:request('PATCH', url, body, options)\n"
"        end,\n"
"\n"
"        --\n"
"        -- <options> see <request>\n"
"        --\n"
"        options = function(self, url, options)\n"
"            check_args(self, 'options')\n"
"            return self:request('OPTIONS', url, nil, options)\n"
"        end,\n"
"\n"
"        --\n"
"        -- <head> see <request>\n"
"        --\n"
"        head = function(self, url, options)\n"
"            check_args(self, 'head')\n"
"            return self:request('HEAD', url, nil, options)\n"
"        end,\n"
"        --\n"
"        -- <delete> see <request>\n"
"        --\n"
"        delete = function(self, url, options)\n"
"            check_args(self, 'delete')\n"
"            return self:request('DELETE', url, nil, options)\n"
"        end,\n"
"\n"
"        --\n"
"        -- <trace> see <request>\n"
"        --\n"
"        trace = function(self, url, options)\n"
"            check_args(self, 'trace')\n"
"            return self:request('TRACE', url, nil, options)\n"
"        end,\n"
"\n"
"        --\n"
"        -- <connect> see <request>\n"
"        --\n"
"        connect = function(self, url, options)\n"
"            check_args(self, 'connect')\n"
"            return self:request('CONNECT', url, nil, options)\n"
"        end,\n"
"\n"
"        --\n"
"        -- <stat> - this function returns a table with many values of statistic.\n"
"        --\n"
"        -- Returns {\n"
"        --\n"
"        --  active_requests - this is number of currently executing requests\n"
"        --\n"
"        --  sockets_added -\n"
"        --  this is a total number of added sockets into libev loop\n"
"        --\n"
"        --  sockets_deleted -\n"
"        --  this is a total number of deleted sockets from libev\n"
"        --                      loop\n"
"        --\n"
"        --  total_requests - this is a total number of requests\n"
"        --\n"
"        --  http_200_responses -\n"
"        --  this is a total number of requests which have\n"
"        --              returned a code HTTP 200\n"
"        --\n"
"        --  http_other_responses -\n"
"        --      this is a total number of requests which have\n"
"        --      requests not a HTTP 200\n"
"        --\n"
"        --  failed_requests - this is a total number of requests which have\n"
"        --      failed (included systeme erros, curl errors, HTTP\n"
"        --      errors and so on)\n"
"        --  }\n"
"        --  or error()\n"
"        --\n"
"        stat = function(self)\n"
"            return self.curl:stat()\n"
"        end,\n"
"\n"
"    },\n"
"}\n"
"\n"
"--\n"
"-- Export\n"
"--\n"
"local http_default = http_new()\n"
"local this_module = { new = http_new, }\n"
"\n"
"local function http_default_wrap(fname)\n"
"    return function(...) return http_default[fname](http_default, ...) end\n"
"end\n"
"\n"
"for _, name in ipairs({ 'get', 'delete', 'trace', 'options', 'head',\n"
"                     'connect', 'post', 'put', 'patch', 'request'}) do\n"
"    this_module[name] = http_default_wrap(name)\n"
"end\n"
"\n"
"package.loaded['http.client'] = this_module\n"
""
;
