const char pwd_lua[] =
"local ffi   = require('ffi')\n"
"local errno = require('errno')\n"
"\n"
"-- GID_T, UID_T and TIME_T are, essentially, `integer types`.\n"
"-- http://pubs.opengroup.org/onlinepubs/009695399/basedefs/sys/types.h.html\n"
"ffi.cdef[[\n"
"    typedef int uid_t;\n"
"    typedef int gid_t;\n"
"    typedef long time_t;\n"
"]]\n"
"\n"
"-- POSIX demands to have three fields in struct group:\n"
"-- http://pubs.opengroup.org/onlinepubs/009695399/basedefs/grp.h.html\n"
"-- char   *gr_name The name of the group.\n"
"-- gid_t   gr_gid  Numerical group ID.\n"
"-- char  **gr_mem  Pointer to a null-terminated array of character pointers to\n"
"--                 member names.\n"
"--\n"
"-- So we'll extract only them.\n"
"ffi.cdef[[\n"
"    struct group {\n"
"        char    *gr_name;    /* group name */\n"
"        char    *gr_passwd;  /* group password */\n"
"        gid_t    gr_gid;     /* group id */\n"
"        char   **gr_mem;     /* group members */\n"
"    };\n"
"]]\n"
"\n"
"-- POSIX demands to have five fields in struct group:\n"
"-- char    *pw_name   User's login name.\n"
"-- uid_t    pw_uid    Numerical user ID.\n"
"-- gid_t    pw_gid    Numerical group ID.\n"
"-- char    *pw_dir    Initial working directory.\n"
"-- char    *pw_shell  Program to use as shell.\n"
"--\n"
"-- So we'll extract only them.\n"
"if ffi.os == 'OSX' or ffi.os == 'BSD' then\n"
"    ffi.cdef[[\n"
"        struct passwd {\n"
"            char    *pw_name;    /* user name */\n"
"            char    *pw_passwd;  /* encrypted password */\n"
"            uid_t    pw_uid;     /* user uid */\n"
"            gid_t    pw_gid;     /* user gid */\n"
"            time_t   pw_change;  /* password change time */\n"
"            char    *pw_class;   /* user access class */\n"
"            char    *pw_gecos;   /* Honeywell login info */\n"
"            char    *pw_dir;     /* home directory */\n"
"            char    *pw_shell;   /* default shell */\n"
"            time_t   pw_expire;  /* account expiration */\n"
"            int      pw_fields;  /* internal: fields filled in */\n"
"        };\n"
"    ]]\n"
"else\n"
"    ffi.cdef[[\n"
"        struct passwd {\n"
"            char *pw_name;   /* username */\n"
"            char *pw_passwd; /* user password */\n"
"            int   pw_uid;    /* user ID */\n"
"            int   pw_gid;    /* group ID */\n"
"            char *pw_gecos;  /* user information */\n"
"            char *pw_dir;    /* home directory */\n"
"            char *pw_shell;  /* shell program */\n"
"        };\n"
"    ]]\n"
"end\n"
"\n"
"ffi.cdef[[\n"
"    uid_t          getuid();\n"
"    struct passwd *getpwuid(uid_t uid);\n"
"    struct passwd *getpwnam(const char *login);\n"
"    void           endpwent();\n"
"    struct passwd *getpwent();\n"
"    void           setpwent();\n"
"\n"
"    gid_t          getgid();\n"
"    struct group  *getgrgid(gid_t gid);\n"
"    struct group  *getgrnam(const char *group);\n"
"    struct group  *getgrent();\n"
"    void           endgrent();\n"
"    void           setgrent();\n"
"]]\n"
"\n"
"local function _getpw(uid)\n"
"    local pw = nil\n"
"    errno(0)\n"
"    if type(uid) == 'number' then\n"
"        pw = ffi.C.getpwuid(uid)\n"
"    elseif type(uid) == 'string' then\n"
"        pw = ffi.C.getpwnam(uid)\n"
"    else\n"
"        error(\"Bad type of uid (expected 'string'/'number')\", 2)\n"
"    end\n"
"    return pw\n"
"end\n"
"\n"
"local function _getgr(gid)\n"
"    local gr = nil\n"
"    errno(0)\n"
"    if type(gid) == 'number' then\n"
"        gr = ffi.C.getgrgid(gid)\n"
"    elseif type(gid) == 'string' then\n"
"        gr = ffi.C.getgrnam(gid)\n"
"    else\n"
"        error(\"Bad type of gid (expected 'string'/'number')\", 2)\n"
"    end\n"
"    return gr\n"
"end\n"
"\n"
"local pwgr_errstr = \"get%s* failed [errno %d]: %s\"\n"
"\n"
"local function getgr(gid)\n"
"    if gid == nil then\n"
"        gid = tonumber(ffi.C.getgid())\n"
"    end\n"
"    local gr = _getgr(gid)\n"
"    if gr == nil then\n"
"        if errno() ~= 0 then\n"
"            error(pwgr_errstr:format('pw', errno(), errno.strerror()), 2)\n"
"        end\n"
"        return nil\n"
"    end\n"
"    local gr_mem, group_members = gr.gr_mem, {}\n"
"    local i = 0\n"
"    while true do\n"
"        local member = gr_mem[i]\n"
"        if member == nil then\n"
"            break\n"
"        end\n"
"        table.insert(group_members, ffi.string(member))\n"
"        i = i + 1\n"
"    end\n"
"    local group = {\n"
"        id      = tonumber(gr.gr_gid),\n"
"        name    = ffi.string(gr.gr_name),\n"
"        members = group_members,\n"
"    }\n"
"    return group\n"
"end\n"
"\n"
"local function getpw(uid)\n"
"    if uid == nil then\n"
"        uid = tonumber(ffi.C.getuid())\n"
"    end\n"
"    local pw = _getpw(uid)\n"
"    if pw == nil then\n"
"        if errno() ~= 0 then\n"
"            error(pwgr_errstr:format('pw', errno(), errno.strerror()), 2)\n"
"        end\n"
"        return nil\n"
"    end\n"
"    local user = {\n"
"        name    = ffi.string(pw.pw_name),\n"
"        id      = tonumber(pw.pw_uid),\n"
"        group   = getgr(pw.pw_gid),\n"
"        workdir = ffi.string(pw.pw_dir),\n"
"        shell   = ffi.string(pw.pw_shell),\n"
"    }\n"
"    return user\n"
"end\n"
"\n"
"local function getpwall()\n"
"    errno(0)\n"
"    ffi.C.setpwent()\n"
"    if errno() ~= 0 then\n"
"        return nil\n"
"    end\n"
"    local pws = {}\n"
"    while true do\n"
"        local pw = ffi.C.getpwent()\n"
"        if pw == nil then\n"
"            break\n"
"        end\n"
"        table.insert(pws, getpw(pw.pw_uid))\n"
"    end\n"
"    ffi.C.endpwent()\n"
"    if errno() ~= 0 then\n"
"        return nil\n"
"    end\n"
"    return pws\n"
"end\n"
"\n"
"local function getgrall()\n"
"    errno(0)\n"
"    ffi.C.setgrent()\n"
"    if errno() ~= 0 then\n"
"        return nil\n"
"    end\n"
"    local grs = {}\n"
"    while true do\n"
"        local gr = ffi.C.getgrent()\n"
"        if gr == nil then\n"
"            break\n"
"        end\n"
"        table.insert(grs, getpw(gr.gr_gid))\n"
"    end\n"
"    ffi.C.endgrent()\n"
"    if errno() ~= 0 then\n"
"        return nil\n"
"    end\n"
"    return grs\n"
"end\n"
"\n"
"return {\n"
"    getpw = getpw,\n"
"    getgr = getgr,\n"
"    getpwall = getpwall,\n"
"    getgrall = getgrall,\n"
"}\n"
""
;
